| header macros for Megadrive / Sega Genesis cartridge header

	.macro	vectors

| backup memory types
MEM_NONE = 0
MEM_BATTERY = 1

| get date and version from command line, and set to zero if not specified
#ifndef DATE
 DATE = 0	| DATE = `date "+%Y%m%d"`
#endif

#ifndef VERSION
 VERSION = 0
#endif

	.section .boot
	.globl	_start
_start:				| start of ROM
	.long	0xFFFFFF00	|  0 reset SP
	.long	COLD_START	|  1 reset PC

	.long	BUS_ERROR	|  2 bus error
	.long	ADDR_ERROR	|  3 address error
	.long	ILLEGAL_INSTR	|  4 illegal instruction
	.long	DIV_ZERO	|  5 divide by zero
	.long	CHK_EXCEPTION	|  6 CHK exception
	.long	TRAP_EXCEPTION	|  7 TRAP exception
	.long	RTE		|  8 privilege violation
	.long	TRACE_EXCEPTION	|  9 trace exception
	.long	LINEA_EXCEPTION	| 10 line 1010 emulator
	.long	LINEF_EXCEPTION	| 11 line 1111 emulator
	.long	RTE		| 12 (unassigned, reserved)
	.long	RTE		| 13 coprocessor protocol violation
	.long	RTE		| 14 format error
	.long	RTE		| 15 uninitialized interrupt

	.long	RTE,RTE,RTE,RTE	| 16-19 (unassigned, reserved)
	.long	RTE,RTE,RTE,RTE	| 20-23 (unassigned, reserved)
	.long	RTE		| 24 suprious interrupt
	.long	RTE		| 25 level 1 (lowest priority)
	.long	EXT_INTERRUPT	| 26 level 2 interrupt = EXT
	.long	RTE		| 27 level 3 interrupt
	.long	HSYNC_INTERRUPT	| 28 level 4 interrupt = H-sync
	.long	RTE		| 29 level 5 interrupt
	.long	VSYNC_INTERRUPT	| 30 level 6 interrupt = V-sync
	.long	RTE		| 31 level 7 (highest priority)

	.long	RTE,RTE,RTE,RTE	| 32-35 TRAP #0-#3
	.long	RTE,RTE,RTE,RTE	| 36-39 TRAP #4-#7
	.long	RTE,RTE,RTE,RTE	| 40-43 TRAP #8-#11
	.long	RTE,RTE,RTE	| 44-46 TRAP #12-#14
	.long	TRAP_15		| 47 TRAP #15

	.long	RTE,RTE,RTE,RTE	| 48-51 (unassigned, reserved)
	.long	RTE,RTE,RTE,RTE	| 52-55 (unassigned, reserved)
	.long	RTE,RTE,RTE,RTE	| 56-59 (unassigned, reserved)
	.long	RTE,RTE,RTE,RTE	| 60-63 (unassigned, reserved)

	.endm


	.macro	ramaddrs
	.long	_start			| start of ROM
	.long	_romend			| end of ROM
RAMspec:
	.long	_data			| start of RAM
	.long	_ramend			| end of RAM
	.endm


	.macro	memory	type,size
 .if \type == MEM_BATTERY
	| battery-backed RAM
	| size = 0x2000 for 8K, 0x8000 for 32K (assuming 8-bit half-word RAM)
	.ascii	"RA"			| backup memory info - battery RAM
	.word	0xF820
	.long	BackupRAM+1		| start addresss
	.long	BackupRAM+\size*2-1	| end address
| .elseif \type == MEM_
	| other types go here
 .else
	| no backup memory
	.ascii	"            "		| backup memory info - none
 .endif
	.endm


	.macro	date1	date
 .if \date == 0
	.ascii	"        "
 .else
	.byte	\date/10000000 + '0'	| year
	.byte	(\date - \date/10000000*10000000)/1000000 + '0'
	.byte	(\date - \date/1000000*1000000)/100000 + '0'
	.byte	(\date - \date/100000*100000)/10000 + '0'
	.ascii	"."			| month
MONTH = (\date - \date/10000*10000)/100
 .if MONTH == 1
	.ascii	"JAN"
 .elseif MONTH == 2
	.ascii	"FEB"
 .elseif MONTH == 3
	.ascii	"MAR"
 .elseif MONTH == 4
	.ascii	"APR"
 .elseif MONTH == 5
	.ascii	"MAY"
 .elseif MONTH == 6
	.ascii	"JUN"
 .elseif MONTH == 7
	.ascii	"JUL"
 .elseif MONTH == 8
	.ascii	"AUG"
 .elseif MONTH == 9
	.ascii	"SEP"
 .elseif MONTH == 10
	.ascii	"OCT"
 .elseif MONTH == 11
	.ascii	"NOV"
 .elseif MONTH == 12
	.ascii	"DEC"
 .else
	.ascii	"   "
 .endif
 .endif
	.endm


	.macro	version	ver
_VersionID:
 	.globl _VersionID
 .if \ver == 0
	.ascii	"    "
 .else
	.ascii	"v"			| version comment
	.byte	\ver/100+'0'
	.byte	(\ver - \ver/100*100)/10 + '0'
	.byte	(\ver - \ver/10*10) + '0'
 .endif
	.ascii	"    "
	.endm


	.macro	date2	date
 .if \date == 0
	.ascii	"          "
 .else
	.byte	\date/10000000 + '0'	| DATE comment
	.byte	(\date - \date/10000000*10000000)/1000000 + '0'
	.byte	(\date - \date/1000000*1000000)/100000 + '0'
	.byte	(\date - \date/100000*100000)/10000 + '0'
	.ascii	"-"
	.byte	(\date - \date/10000*10000)/1000 + '0'
	.byte	(\date - \date/1000*1000)/100 + '0'
	.ascii	"-"
	.byte	(\date - \date/100*100)/10 + '0'
	.byte	(\date - \date/10*10) + '0'
 .endif
	.ascii	"                      " | more comment
	.endm


	.macro	bootcode

| weak symbols for exception vectors that can be overriden elsewhere

	.weak	BUS_ERROR, ADDR_ERROR, ILLEGAL_INSTR, DIV_ZERO
	.weak	CHK_EXCEPTION, TRAP_EXCEPTION, TRACE_EXCEPTION
	.weak	LINEA_EXCEPTION, LINEF_EXCEPTION, EXT_INTERRUPT
	.weak	HSYNC_INTERRUPT, VSYNC_INTERRUPT, TRAP_15

BUS_ERROR:
ADDR_ERROR:
	addq.l	#8,%a7			| clean up stack frame
ILLEGAL_INSTR:
DIV_ZERO:
CHK_EXCEPTION:
TRAP_EXCEPTION:
TRACE_EXCEPTION:
LINEA_EXCEPTION:
LINEF_EXCEPTION:
EXT_INTERRUPT:
HSYNC_INTERRUPT:
VSYNC_INTERRUPT:
TRAP_15:
RTE:
	rte				| return from exception

COLD_START:
	lea	InitRegs, %a1		| A1 = address of constants table
	lea	0xC00000, %a5		| A5 = C00000 VDP data
	lea	4(%a5), %a6		| A6 = C00004 VDP control / status
	move.w	(%a6), %d0		| read VDP status (interrupt acknowledge?)

	| === INITIALIZE TMSS ===

	move.b	0xA10001, %d0		| read hardware version
	andi.b	#0x0F, %d0		| TMSS present if low 4 bits != 0
	beq.b	1f			| branch if no TMSS
	move.l	#0x53454741, 0xA14000	| move "SEGA" to the TMSS register
1:
	| === INITIALIZE USP ===

	moveq	#0, %d0			| D0 = 0
 	movea.l	%d0, %a0		| A6 = 0 (== alias to end of RAM + 1)
 	move	%a0, %usp		| set up USP

	| If this isn't written to the VDP within a few dozen cycles of startup,
	| the OS X version of SDLMESS makes the window small.  Why?  Who the hell knows.
	| It's annoying and flickery when it happens, so let's not let it happen.

	move.w	#0x8C81, (%a6)		| reg 12 = mode register 4: 40 cell horizontal mode, no interlace
 
	| === INITIALIZE VDP REGISTERS ===

	moveq	#24-1, %d1		| length of video initialization block
	move.w	#0x8000, %d7		| first register number
1:	move.b	(%a1)+, %d7		| get next video control byte (high word is command!)
	move.w	%d7, (%a6)		| C00004 send write register command to VDP
	add.w	#0x0100, %d7		| point to next VDP register
	dbra	%d1, 1b			| loop for rest of block

	| === INITIALIZE RAM ===
	| (this also initializes gcc's .bss section)

	move.l	RAMspec, %a0		| get start of RAM
 .if 0
	move.l	RAMspec+4, %d1
	sub.l	RAMspec, %d1		| D1 = size of RAM - 1
	lsr.l	#2, %d1			| D1 = size of RAM in longwords - 1
 .else
	move.w	#0x4000-1, %d1		| get size of RAM in longwords
 .endif
1:	move.l	%d0, (%a0)+
	dbra	%d1, 1b

	| === INITIALIZE VRAM ===

	| DMA is now set up for 65535-byte fill of VRAM, let's do it
	move.l	#0x40000080, (%a6)	| C00004 = VRAM write to $0000 with CD5 bit set
	move.w	%d0, (%a5)		| C00000 = write zero to VRAM (starts DMA fill)

1:	move.w	(%a6), %d4		| C00004 read VDP status
	btst	#1, %d4			| test DMA busy flag
	bne.b	1b			| loop while DMA busy

	| === INITIALIZE CRAM ===

	move.l	#0x81048F02, (%a6)	| C00004 reg 1 = 0x04, reg 15 = 0x02: blank, auto-increment=2
	move.l	#0xC0000000, (%a6)	| C00004 write CRAM address $0000
	moveq	#32-1, %d1		| loop for 32 CRAM registers
1:	move.l	%d0, (%a5)		| C00000 clear CRAM register
	dbra	%d1, 1b

	| === INITIALIZE VSRAM ===

	move.l	#0x40000010, (%a6)	| C00004 VSRAM write address $0000
	moveq	#20-1, %d1		| loop for 20 VSRAM registers
1:	move.l	%d0, (%a5)		| C00000 clear VSRAM register
	dbra	%d1, 1b

	| === INITIALIZE PSG ===

	moveq	#4-1, %d1		| loop for 4 PSG registers
1:	move.b	(%a1)+, 0x0011(%a5)	| C00011 copy PSG initialization commands
	dbra	%d1, 1b

	| === C INITIALIZED DATA ===

	lea	_etext, %a0		| initialization data in ROM
	lea	_data, %a1		| start of initialized RAM
	lea	_edata, %a2		| end of initialized RAM
1:
	cmpa.l	%a1, %a2		| at end of RAM?
	beq.s	2f			| exit loop if end of RAM
	move.b	(%a0)+, (%a1)+		| copy next byte
	bra.s	1b			| loop for rest of initialized data
2:
|	bsr.w	libc_init		| initialize C libraries
|	bsr.w	frame_dummy		| C++ initialization stuff
|	bsr.w	__do_global_ctors_aux	| C++ initialization stuff
	bsr.w	main			| start main program
|	bsr.w	__do_global_dtors_aux	| C++ finishing stuff
	bra.b	.			| loop forever if main program exits

|-----------------------------------------------------------------------
|	VRAM layout:
|
|	0000 - FFFF pattern generator table     (all of VRAM)
|	C000 - CFFF scroll A name table base    (max size 1000)
|	D800 - DFFF sprite attribute table base (max size 0A00)
|	E000 - EFFF scroll B name table base    (max size 1000)
|	F000 - F7FF window name table base      (max size 0800)
|	FC00 - FFFF H-scroll table base         (max size 0400)
|-----------------------------------------------------------------------

InitRegs:
	| VDP register initialization (24 bytes)
	.byte	0x04	| reg  0 = mode register 1: no H interrupt
	.byte	0x14	| reg  1 = mode register 2: blanked, no V interrupt, DMA enable
	.byte	0x30	| reg  2 = name table base for scroll A: $C000
	.byte	0x3C	| reg  3 = name table base for window:   $F000
	.byte	0x07	| reg  4 = name table base for scroll B: $E000
	.byte	0x6C	| reg  5 = sprite attribute table base:  $D800
	.byte	0x00	| reg  6 = unused register: $00
	.byte	0x00	| reg  7 = background color: $00 = plt 0 color 0
	.byte	0x00	| reg  8 = unused register: $00
	.byte	0x00	| reg  9 = unused register: $00
	.byte	0xFF	| reg 10 = H interrupt register: $FF (esentially off)
	.byte	0x00	| reg 11 = mode register 3: disable ext int, full H/V scroll
	.byte	0x81	| reg 12 = mode register 4: 40 cell horizontal mode, no interlace
	.byte	0x37	| reg 13 = H scroll table base: $FC00
	.byte	0x00	| reg 14 = unused register: $00
	.byte	0x01	| reg 15 = auto increment: $01
	.byte	0x01	| reg 16 = scroll size: V=32 cell, H=64 cell
	.byte	0x00	| reg 17 = window H position: $00
	.byte	0x00	| reg 18 = window V position: $00
	.byte	0xFF	| reg 19 = DMA length count low:   $xxFF
	.byte	0xFF	| reg 20 = DMA length count high:  $FFxx
	.byte	0x00	| reg 21 = DMA source address low: $xxxx00
	.byte	0x00	| reg 22 = DMA source address mid: $xx00xx
	.byte	0x80	| reg 23 = DMA source address high: VRAM fill, addr = $00xxxx

	| PSG initialization: set all channels to minimum volume
	.byte	0x9F,0xBF,0xDF,0xFF

	.endm
