include $(MDSDK)/site.mk

.PHONY: all
all: $(NAME).bin

dep: # this forces the gcc dependencies file to be rebuilt
	$(CC) ${CPPFLAGS} -E -MM *.c > .depend
.depend:
	$(CC) ${CPPFLAGS} -E -MM *.c > .depend
include .depend

$(NAME).bin: $(OBJS)
#note: let gcc call ld to ensure that you get the right runtime libraries
	$(CC) $(CFLAGS) $(LDFLAGS) -o $(NAME).elf $(OBJS) $(MDSDK)/libio.o
	$(OBJCOPY) -j .text -j .data -O binary $(NAME).elf $(NAME).bin
	$(OBJDUMP) -d $(NAME).elf > $(NAME).diss
	-rm -f $(NAME).elf

header.o: header.S
	#$(DATE)
# the expr command is to strip leading zeros from version numbers
# so they don't look like octal numbers to the assembler
	$(CC) ${CPPFLAGS} $(CFLAGS) -DDATE=$(DATE) -DVERSION=$(shell expr $(VERSION)) -Wa,"-I$(MDSDK)" -c header.S -o header.o

.PHONY: info
info: $(NAME).bin
	@grep "_romend" $(NAME).map | sed 's/^.*\(0x[0-9A-Fa-f]*\).*$$/ROM end = \1/'
	@grep "_ramend" $(NAME).map | sed 's/^.*\(0x[0-9A-Fa-f]*\).*$$/RAM end = \1/'

.PHONY: run
run: $(NAME).bin
	cd $(EMUDIR) ; $(EMU) $(EMUFLAGS) $(CURDIR)/$(NAME).bin >/dev/null 2>/dev/null &

.PHONY: mess
mess: run

.PHONY: disk
disk: $(NAME).bin
	$(MAKESMD) $(NAME).bin >/dev/null
	if [ -d $(FLOPPY) ] ; then cp $(SMDNAME) $(FLOPPY) ; $(EJECT) ; fi

